#!/usr/bin/perl


# dnswl.org - Protect against false positives
#
# rsync the Postfix data and (optionally) verify it's
# authenticity through GnuPG. 
#
# Copyright dnswl.org, Originally written by Matthias Leisi
# Licensed under the GPL v2.
#
# $Id: postfix-update-dnswl.pl,v 1.4 2006/11/27 19:02:52 mleisi Exp $
#

use strict;
use File::Copy;
require 'dnswl-lib.pl';

my $conf = findConfig();

if (!(-f $conf)) {
	print "Configuration file $conf not found\n";
	exit 1;
}
{ package dnswl; do "$conf"; }

print "Using configuration file $conf\n\n" if ($dnswl::debug);


##################################################################
#
# You should not need to change things below. 
#
my @args = ();
if ($dnswl::postfix_getheader == 1) {
	my $tmpfile = sprintf('%s/%s', $dnswl::tmpdir, $dnswl::headerfile);
	my $outfile = sprintf('%s/%s', $dnswl::postfix_dir, $dnswl::headerfile);
	my $signaturefile = sprintf('%s.asc', $dnswl::headerfile);

	# 1. Get the file
	if (!(rsync($dnswl::headerfile, $tmpfile) == 0)) {
		print "rsync of dnswl.org data for Postfix (headerfile) failed\n";
		exit 1;
	}

	my $verify = verify($tmpfile, $signaturefile);

	if ($verify == 1) {
		# 2. Substitute 'REPLACEME' with $whitelistheader
		if (length($dnswl::whitelistheader) > 0) {
			print "Inserting $dnswl::whitelistheader\n" if ($dnswl::debug);
			open(IN, "$tmpfile") or die sprintf('Could not open %s for reading: %s', $tmpfile, $!);
			open(OUT, "> $outfile") or die sprintf('Could not open %s for writing: %s', $outfile, $!);
			while (<IN>) {
				my $line = $_;
				next if ($line =~ /^#/);
				$line =~ s/X-REPLACEME/$dnswl::whitelistheader/;
				print OUT $line;
			}
			close(IN);
			close(OUT);
		}
	} else {
		print "PGP VERIFICATION FAILED!\n";
		print "You may want to verify the data file: $tmpfile and the signature file: $signaturefile\n";
		exit $verify;
	}
}

if ($dnswl::postfix_getpermit == 1) {
	my $tmpfile = sprintf('%s/%s', $dnswl::tmpdir, $dnswl::permitfile);
	my $outfile = sprintf('%s/%s', $dnswl::postfix_dir, $dnswl::permitfile);
	my $signaturefile = sprintf('%s.asc', $dnswl::permitfile);

	# 1. Get the file
	if (!(rsync($dnswl::permitfile, $tmpfile) == 0)) {
		print "rsync of dnswl.org data for Postfix (permitfile) failed\n";
	}
	
	my $verify = verify($tmpfile, $signaturefile);

	if ($verify == 1) {
		# 2. Filter according to $permitregex
		print "Applying $dnswl::permitregex to $dnswl::permitaction\n" if ($dnswl::debug);
		open(IN, "$tmpfile") or die sprintf('Could not open %s for reading: %s', $tmpfile, $!);
		open(OUT, "> $outfile") or die sprintf('Could not open %s for writing: %s', $outfile, $!);
		while (<IN>) {
			my $line = $_;
			next if ($line =~ /^#/);
			if (length($dnswl::permitregex) > 0) {
				next if (!($line =~ /$dnswl::permitregex/));
			}
			if (length($dnswl::permitaction) > 0) {
				$line =~ s/permit_auth_destination/$dnswl::permitaction/;
			}
			
			print OUT $line;
		}
		close(IN);
		close(OUT);
	} else {
		print "PGP VERIFICATION FAILED!\n";
		print "You may want to verify the data file: $tmpfile and the signature file: $signaturefile\n";
		exit $verify;
	}		
}

# EOF
