#!/usr/bin/perl

# dnswl.org - Protect against false positives
#
# rsync the rbldnsd data and (optionally) verify it's
# authenticity through GnuPG
#
# Copyright dnswl.org, Originally written by Matthias Leisi
# Licensed under the GPL v2.
#
# $Id: rbldnsd-update-dnswl.pl,v 1.4 2006/11/26 21:04:09 mleisi Exp $
#

use strict;
use File::Copy;
require 'dnswl-lib.pl';

my $conf = findConfig();

if (!(-f $conf)) {
	print "Configuration file $conf not found\n";
	exit 1;
}
{ package dnswl; do "$conf"; }

print "Using configuration file $conf\n\n" if ($dnswl::debug);

if (!(-d $dnswl::rbldnsd_datadir)) {
	print "Configuration error in $conf:\$rbldnsd_datadir=$dnswl::rbldnsd_datadir\n\tDirectory does not exist\n";
	exit 1;
}

if (!(-d $dnswl::tmpdir)) {
	print "Configuration error in $conf:\$tmpdir=$dnswl::tmpdir\n\tDirectory does not exist\n";
	exit 1;
}

my $tmpfile = sprintf('%s/%s.tmp', $dnswl::tmpdir, $dnswl::rbldnsd_filename);
my $finalfile = sprintf('%s/%s', $dnswl::rbldnsd_datadir, $dnswl::rbldnsd_filename);
my $signaturefile = sprintf('%s.asc', $dnswl::rbldnsd_filename);

if (!(rsync($dnswl::rbldnsd_filename, $tmpfile) == 0)) {
	print "rsync of dnswl.org data for rbldnsd failed.\n";
	exit 1;
}

my $verify = verify($tmpfile, $signaturefile);

if ($verify == 1) {
	print "Moving $tmpfile to $finalfile\n" if ($dnswl::debug);
	copy($tmpfile, $finalfile) or die "Copy of $tmpfile to $finalfile failed: $!\n";
	exit 0;
} else {
	print "PGP VERIFICATION FAILED!\n";
	print "You may want to verify the data file: $tmpfile and the signature file: $signaturefile\n";
	exit $verify;
}

# EOF
